<?php

declare(strict_types=1);


namespace App\Infrastructure\Arbitration\Inner;


use App\Arbitration\Inner\ArbitrationChain;
use App\Arbitration\Inner\ChainProviderInterface;
use App\Exchanges\ExchangeInterface;
use App\Exchanges\MarketInterface;
use MongoDB\Client;
use MongoDB\Collection;

class ChainProvider implements ChainProviderInterface
{

    private Collection $collection;

    public function __construct(
        Client $client
    )
    {
        $this->collection = $client->selectCollection('arbitry', 'inner_arbitration_chains');
    }

    /**
     * @inheritDoc
     */
    public function getChainsForExchange(ExchangeInterface $exchange): array
    {
        $rows = $this->collection->find(['exchange' => $exchange->getName()]);

        $markets = $exchange->getMarkets();

        $chains = [];
        foreach ($rows as $row) {
            $chainMarkets = [];
            foreach ($row['chain'] ?? [] as $chainData) {
                foreach ($markets as $market) {
                    if ((string)$market->getPair() === $chainData) {
                        $chainMarkets[] = $market;
                    }
                }
            }

            if (!empty($chainMarkets)) {
                $chains[] = new ArbitrationChain(...$chainMarkets);
            }
        }

        return $chains;
    }

    protected function calculateChainHash(ExchangeInterface $exchange, ArbitrationChain $chain): string
    {
        return md5(
            $exchange->getName()
            . ';'
            . implode(
                ';',
                array_map(
                    static function (MarketInterface $market): string
                    {
                        return (string)$market->getPair();
                    },
                    $chain->getMarkets()
                )
            )
        );
    }

    public function addChain(ExchangeInterface $exchange, ArbitrationChain $chain): void
    {
        $this
            ->collection
            ->insertOne([
                '_id' => $this->calculateChainHash($exchange, $chain),
                'exchange' => $exchange->getName(),
                'chain' => array_map(
                    static function (MarketInterface $market): string
                    {
                        return (string)$market->getPair();
                    },
                    $chain->getMarkets()
                )
            ]);
    }

    public function removeChain(ExchangeInterface $exchange, ArbitrationChain $chain): void
    {
        $this
            ->collection
            ->deleteOne([
                '_id' => $this->calculateChainHash($exchange, $chain)
            ]);
    }
}